/* ------------------------------------------------------------------------*/
/*                                                                         */
/*   SYSTEM.H                                                              */
/*                                                                         */
/*   Copyright (c) Borland International 1991                              */
/*   All Rights Reserved.                                                  */
/*                                                                         */
/*   defines the classes THWMouse, TMouse, TEventQueue, TDisplay,          */
/*   TScreen, and TSystemError                                             */
/*                                                                         */
/* ------------------------------------------------------------------------*/


#if !defined( __EVENT_CODES )
#define __EVENT_CODES

/* Event codes */

const evMouseDown = 0x0001;
const evMouseUp   = 0x0002;
const evMouseMove = 0x0004;
const evMouseAuto = 0x0008;
const evKeyDown   = 0x0010;
const evCommand   = 0x0100;
const evBroadcast = 0x0200;

/* Event masks */

const evNothing   = 0x0000;
const evMouse     = 0x000f;
const evKeyboard  = 0x0010;
const evMessage   = 0xFF00;

/* Mouse button state masks */

const mbLeftButton  = 0x01;
const mbRightButton = 0x02;

#endif  // __EVENT_CODES


#if defined( Uses_TEvent ) && !defined( __TEvent )
#define __TEvent

struct MouseEventType
{
    uchar buttons;
    Boolean doubleClick;
    TPoint where;
};

class THWMouse
{

protected:

    THWMouse();
    THWMouse( const THWMouse& ) {};
    ~THWMouse();

    static void show();
    static void hide();
    static void hide( const TRect& rect );

    static void setRange( ushort, ushort );
    static void getEvent( MouseEventType& );
#if defined(__TV4WATCOMDOS__)
    static void registerHandler( unsigned, void _loadds (far*)(int,int,int,int) );
#else
    static void registerHandler( unsigned, void (*)() );
#endif
    static Boolean present();

    static void suspend();
    static void resume();
    static void inhibit();

protected:

    static uchar buttonCount;

private:

    static Boolean handlerInstalled;
    static Boolean noMouse;

};

inline Boolean THWMouse::present()
{
    return Boolean( buttonCount != 0 );
}

inline void THWMouse::inhibit()
{
    noMouse = True;
}

class TMouse : public THWMouse
{

public:

    TMouse();
    ~TMouse();

    static void show();
    static void hide();
    static void hide( const TRect& rect ); // hides the Mouse in a Rectagular Area.

    static void setRange( ushort, ushort );

    static void getEvent( MouseEventType& );
#if defined(__TV4WATCOMDOS__)
    static void registerHandler( unsigned, void _loadds (far)(int,int,int,int) );
#else
    static void registerHandler( unsigned, void (*)() );
#endif
    static Boolean present();

    static void suspend() { THWMouse::suspend(); }
    static void resume(); // { THWMouse::resume(); show(); } BC bug

};

#if defined (__TV4WATCOMDOS__)
void _loadds far globalMouseInt( int max, int mbx, int mcx, int mdx );
#endif


inline void TMouse::show()
{
    THWMouse::show();
}

inline void TMouse::resume()
{
  THWMouse::resume(); show();
};

inline void TMouse::hide()
{
    THWMouse::hide();
}

inline void TMouse::hide( const TRect& rect )
{
    THWMouse::hide( rect );
}

inline void TMouse::setRange( ushort rx, ushort ry )
{
    THWMouse::setRange( rx, ry );
}

inline void TMouse::getEvent( MouseEventType& me )
{
    THWMouse::getEvent( me );
}

#ifdef __TV4WATCOMDOS__
inline void TMouse::registerHandler( unsigned mask, void _loadds (far *func)(int,int,int,int) )
#else
inline void TMouse::registerHandler( unsigned mask, void (*func)() )
#endif
{
    THWMouse::registerHandler( mask, func );
}

inline Boolean TMouse::present()
{
    return THWMouse::present();
}

#pragma pack(1)
struct CharScanType
{
    uchar charCode;
    uchar scanCode;
};
#pragma pack()

struct KeyDownEvent
{
    union
        {
        ushort keyCode;
        CharScanType charScan;
        };
};

struct MessageEvent
{
    ushort command;
    union
        {
        void *infoPtr;
        long infoLong;
        ushort infoWord;
        short infoInt;
        uchar infoByte;
        char infoChar;
        };
};

struct TEvent
{
    ushort what;
    unsigned long time;
    union
    {
        MouseEventType mouse;
        KeyDownEvent keyDown;
        MessageEvent message;
    };
    void getMouseEvent();
    void getKeyEvent();
};

#endif  // Uses_TEvent

#if defined( Uses_TEventQueue ) && !defined( __TEventQueue )
#define __TEventQueue

struct TEventWithSemaphore
{
    volatile short semaphore;
    volatile TEvent event;
};

class TEventQueue
{
public:
    TEventQueue();
    ~TEventQueue();

    static void getMouseEvent( TEvent& );
    static void suspend();
    static void resume();

    friend class TView;
    friend void genRefs();
    friend class TProgram;
    friend class THWMouse;
    static unsigned long doubleDelay;
    static Boolean mouseReverse;

private:

    static TMouse mouse;
    static void getMouseState( TEvent& );
#if defined( __TV4WATCOMDOS__ )
    static void mouseInt( int max, int mbx, int mcx, int mdx );
    friend void _loadds far globalMouseInt( int,int,int,int);
#else
    static void mouseInt(); // dummy function
#endif
    static void setLast( TEvent& );

    static MouseEventType lastMouse;
    static volatile long curMouseSemaphore;
    static volatile MouseEventType curMouse;

    static MouseEventType downMouse;
    static unsigned long downTicks;

//    static ushort * Ticks;
#if 0
    static TEvent eventQueue[ eventQSize ];
    static TEvent * eventQHead;
    static TEvent * eventQTail;
    static Boolean mouseIntFlag;
    static ushort eventCount;
#endif
    static volatile TEventWithSemaphore eventQueue[ eventQSize ];
    static volatile int eventQHead;
    static volatile int eventQTail;
    static Boolean mouseIntFlag;

    static Boolean mouseEvents;

    static unsigned long repeatDelay;
    static unsigned long autoTicks;
    static unsigned long autoDelay;

#ifdef __TV4OS2__
public:
    static void mouseThread(void* arg);
    static void keyboardThread( void * arg );
    static TEvent keyboardEvent;
    static unsigned char shiftState;
#endif

};

inline void TEvent::getMouseEvent()
{
    TEventQueue::getMouseEvent( *this );
}

#endif  // Uses_TEventQueue

#if defined( Uses_TScreen ) && !defined( __TScreen )
#define __TScreen

#ifdef PROTECT

extern ushort monoSeg;
extern ushort colrSeg;
extern ushort biosSeg;

#endif

class TDisplay
{

public:

    friend class TView;

    enum videoModes
        {
        smBW80      = 0x0002,
        smCO80      = 0x0003,
        smMono      = 0x0007,
        smFont8x8   = 0x0100
        };

    static void clearScreen( uchar, uchar );

    static void setCursorType( ushort );
    static ushort getCursorType();

    static ushort getRows();
    static ushort getCols();

    static void setCrtMode( ushort );
    static ushort getCrtMode();

protected:

    TDisplay() { updateIntlChars(); };
    TDisplay( const TDisplay& ) { updateIntlChars(); };
    ~TDisplay() {};

private:

    static void videoInt();
    static void updateIntlChars();

    static ushort * equipment;
    static uchar * crtInfo;
    static uchar * crtRows;

};

class TScreen : public TDisplay
{

public:

    TScreen();
    ~TScreen();

    static void setVideoMode( ushort mode );
    static void clearScreen();

    static ushort startupMode;
    static ushort startupCursor;
    static ushort screenMode;
    static uchar screenWidth;
    static uchar screenHeight;
    static Boolean hiResScreen;
    static Boolean checkSnow;
    static uchar * screenBuffer;
    static ushort cursorLines;

    static void setCrtData();
    static ushort fixCrtMode( ushort );

    static void suspend();
    static void resume();

};

#endif  // Uses_TScreen

#if defined( Uses_TSystemError ) && !defined( __TSystemError )
#define __TSystemError

class TDrawBuffer;

class TSystemError
{

public:

    TSystemError();
    ~TSystemError();

    static Boolean ctrlBreakHit;

    static void suspend();
    static void resume();
    static short ( *sysErrorFunc )( short, uchar );

private:

    static ushort sysColorAttr;
    static ushort sysMonoAttr;
    static Boolean saveCtrlBreak;
    static Boolean sysErrActive;

    static void swapStatusLine( TDrawBuffer & );
    static ushort selectKey();
    static short sysErr( short, uchar );

    static Boolean inIDE;

//    static const char *const errorString[14];
//    static const char *const *errorString1;
    static const char * const *errorString;
    static const char * sRetryOrCancel;


};

#endif  // Uses_TSystemError

#if defined( Uses_TOS2Handles ) && !defined( __TOS2Handles)
#define __TOS2Handles

#ifdef __TV4OS2__
#define INCL_NOPMAPI
#define INCL_KBD
#define INCL_MOU
#define INCL_VIO
#define INCL_DOSPROCESS
#define INCL_DOSMEMMGR
#define INCL_DOSSEMAPHORES
#define INCL_DOSFILEMGR
#define INCL_DOSERRORS
#define INCL_DOSMISC
#include <os2.h>

// These must not cross a 64K boundary
struct TiledStruct {
  HMOU mouseHandle;
  MOUEVENTINFO mouseInfo;
  NOPTRRECT ptrArea;
  PTRLOC ptrLoc;
  VIOCURSORINFO cursorInfo;
  VIOMODEINFO modeInfo;
  KBDKEYINFO keyboardInfo;
  KBDINFO    keyboardShiftInfo;
  KBDINFO    savedKbdStatus;
};

class TOS2Handles {
public:
  TOS2Handles();
  ~TOS2Handles();
  static void resume();
  static void suspend();
  static void deadEnd();
  static TiledStruct *tiled; // tiled will be allocated with DosAllocMem
  static volatile TID mouseThreadID;
  static volatile TID keyboardThreadID;
  static volatile SEMRECORD evCombined[2];
  static volatile HEV &hevMouse1;
  static volatile HEV &hevKeyboard1;
  static volatile HEV hevKeyboard2;
  static volatile HEV hevMouse2;
  static volatile HMTX hmtxMouse1;
  static volatile HMUX hmuxMaster;
  static volatile int shutDownFlag;
};

#endif // TV4OS2

#ifdef __TV4EMX__

// Interface to the emxmouse.exe TSR

#pragma pack(2)
const EMXQueueSize = 32;
struct EMXMouseEvent {
  unsigned short buttons;
  unsigned short x,y;
};
struct EMXEvent {
  unsigned short semaphore;
  unsigned long  time;
  EMXMouseEvent  mouse;
};
struct EMXSharedRec {
  unsigned long magic;
  volatile unsigned short curMouseSemaphore;
  volatile EMXMouseEvent curMouse;
  volatile unsigned short eventSemaphore;
  volatile EMXEvent eventQueue[EMXQueueSize];
  unsigned short eventQHead;
  unsigned short eventQTail;
  unsigned short buttonCount;
};
#pragma pack()

extern EMXSharedRec* EMXPtr;

#endif

#endif // Uses_TOS2Handles

