/* ------------------------------------------------------------------------*/
/*                                                                         */
/*   TOBJSTRM.H                                                            */
/*                                                                         */
/*   Copyright (c) Borland International 1991                              */
/*   All Rights Reserved.                                                  */
/*                                                                         */
/*   defines the classes TStreamable, TStreamableClass, pstream,           */
/*   ipstream, opstream, iopstream, ifpstream, ofpstream, and fpstream.    */
/*                                                                         */
/* ------------------------------------------------------------------------*/

typedef unsigned P_id_type;

// I really don't know why borland used Cdecl for this module.
// If it _did_ make sense, define UselessCdecl as _Cdecl 
// JS
#define UselessCdecl

/* ------------------------------------------------------------------------*/
/*                                                                         */
/*   class TStreamable                                                     */
/*                                                                         */
/*   This is the base class for all storable objects.  It provides         */
/*   three member functions, streamableName(), read(), and write(), which  */
/*   must be overridden in every derived class.                            */
/*                                                                         */
/* ------------------------------------------------------------------------*/


#if !defined( __fLink_def )
#define __fLink_def
class TStreamableClass;
struct fLink
{
    fLink *f;
    class TStreamableClass *t;
};
#endif

#define __link( s )             \
  extern TStreamableClass s;    \
  static fLink force ## s =     \
    { (fLink *)&force ## s, (TStreamableClass *)&s };

#if defined( Uses_TStreamable ) && !defined( __TStreamable )
#define __TStreamable

class TStreamable
{

    friend class opstream;
    friend class ipstream;

private:

    virtual const char *streamableName() const = 0;

protected:

    virtual void *read( ipstream& ) = 0;
    virtual void write( opstream& ) = 0;

};

#endif  // Uses_TStreamable

/* ------------------------------------------------------------------------*/
/*                                                                         */
/*   class TStreamableClass                                                */
/*                                                                         */
/*   Used internally by TStreamableTypes and pstream.                      */
/*                                                                         */
/* ------------------------------------------------------------------------*/

#if defined( Uses_TStreamableClass ) && !defined( __TStreamableClass )
#define __TStreamableClass

#include <limits.h>

const P_id_type P_id_notFound = UINT_MAX;

typedef TStreamable *(*BUILDER)();
#if 0
#define __DELTA( d ) (FP_OFF((TStreamable *)(d *)1)-1)
#endif
#define __DELTA( d ) (((unsigned) ((TStreamable *)(d *)1) )-1UL)

class TStreamableClass
{

    friend class TStreamableTypes;
    friend class opstream;
    friend class ipstream;

public:

    TStreamableClass( const char *n, BUILDER b, int d );

private:

    const char *name;
    BUILDER build;
    int delta;

};

#endif  // Uses_TStreamableClass

/* ------------------------------------------------------------------------*/
/*                                                                         */
/*   class TStreamableTypes                                                */
/*                                                                         */
/*   Maintains a database of all registered types in the application.      */
/*   Used by opstream and ipstream to find the functions to read and       */
/*   write objects.                                                        */
/*                                                                         */
/* ------------------------------------------------------------------------*/

#if defined( Uses_TStreamableTypes ) && !defined( __TStreamableTypes )
#define __TStreamableTypes

class TStreamableTypes : private TNSSortedCollection
{

public:

    TStreamableTypes();
    ~TStreamableTypes();

    void registerType( const TStreamableClass * );
    const TStreamableClass *lookup( const char * );

    void *operator new( size_t sz ) { return ::operator new( sz ); }
    void *operator new( size_t, void * );

private:

    virtual void *keyOf( void * );
    int compare( void *, void * );

};

#endif  // Uses_TStreamableTypes

/* ------------------------------------------------------------------------*/
/*                                                                         */
/*   class TPWrittenObjects                                                */
/*                                                                         */
/*   Maintains a database of all objects that have been written to the     */
/*   current object stream.                                                */
/*                                                                         */
/*   Used by opstream when it writes a pointer onto a stream to determine  */
/*   whether the object pointed to has already been written to the stream. */
/*                                                                         */
/* ------------------------------------------------------------------------*/

#if defined( Uses_TPWrittenObjects ) && !defined( __TPWrittenObjects )
#define __TPWrittenObjects

class TPWrittenObjects : public TNSSortedCollection
{

    friend class opstream;

public:

    void removeAll() { curId = 0; TNSSortedCollection::removeAll(); }

private:

    TPWrittenObjects();
    ~TPWrittenObjects();

    void registerObject( const void *adr );
    P_id_type find( const void *adr );

    void *keyOf( void * );
    int compare( void *, void * );

    P_id_type curId;

};

/* ------------------------------------------------------------------------*/
/*                                                                         */
/*   class TPWObj                                                          */
/*                                                                         */
/*   Used internally by TPWrittenObjects.                                  */
/*                                                                         */
/* ------------------------------------------------------------------------*/

class TPWObj
{

    friend class TPWrittenObjects;

private:

    TPWObj( const void *adr, P_id_type id );

    const void *address;
    P_id_type ident;

};

#endif  // Uses_TPWrittenObjects

/* ------------------------------------------------------------------------*/
/*                                                                         */
/*   class TPReadObjects                                                   */
/*                                                                         */
/*   Maintains a database of all objects that have been read from the      */
/*   current persistent stream.                                            */
/*                                                                         */
/*   Used by ipstream when it reads a pointer from a stream to determine   */
/*   the address of the object being referred to.                          */
/*                                                                         */
/* ------------------------------------------------------------------------*/

#if defined( Uses_TPReadObjects ) && !defined( __TPReadObjects )
#define __TPReadObjects

class TPReadObjects : public TNSCollection
{

    friend class ipstream;

public:

    void removeAll() { curId = 0; TNSCollection::removeAll(); }

private:

    TPReadObjects();
    ~TPReadObjects();

    void registerObject( const void *adr );
    const void *find( P_id_type id );

    P_id_type curId;

};

#endif  // Uses_TPReadObjects

/* ------------------------------------------------------------------------*/
/*                                                                         */
/*   class pstream                                                         */
/*                                                                         */
/*   Base class for handling streamable objects.                           */
/*                                                                         */
/* ------------------------------------------------------------------------*/

#if defined( Uses_pstream ) && !defined( __pstream )
#define __pstream

#if !defined( __IOSTREAM_H )
#include <iostream.h>
#endif  // __IOSTREAM_H


class TStreamableTypes;

class pstream
{

    friend class TStreamableTypes;

public:

    enum StreamableError { peNotRegistered, peInvalidType };
    enum PointerTypes { ptNull, ptIndexed, ptObject };

    UselessCdecl pstream( streambuf * );
    virtual UselessCdecl ~pstream();

    int UselessCdecl rdstate() const;
    int UselessCdecl eof() const;
    int UselessCdecl fail() const;
    int UselessCdecl bad() const;
    int UselessCdecl good() const;
    void UselessCdecl clear( int = 0 );
    UselessCdecl operator void *() const;
    int UselessCdecl operator ! () const;

    streambuf * UselessCdecl rdbuf() const;

    static void initTypes();

    void UselessCdecl error( StreamableError );
    void UselessCdecl error( StreamableError, const TStreamable& );
    static void registerType( TStreamableClass *ts );

protected:

    UselessCdecl pstream();

    streambuf *bp;
    int state;

    void UselessCdecl init( streambuf * );
    void UselessCdecl setstate( int );

    static TStreamableTypes * types;

};

#endif  // Uses_pstream

/* ------------------------------------------------------------------------*/
/*                                                                         */
/*   class ipstream                                                        */
/*                                                                         */
/*   Base class for reading streamable objects                             */
/*                                                                         */
/* ------------------------------------------------------------------------*/

#if defined( Uses_ipstream ) && !defined( __ipstream )
#define __ipstream

#if !defined( __IOSTREAM_H )
#include <iostream.h>
#endif  // __IOSTREAM_H


class TStreamableClass;

class ipstream : virtual public pstream
{

public:

    UselessCdecl ipstream( streambuf * );
    UselessCdecl ~ipstream();

    streampos UselessCdecl tellg();
    ipstream& UselessCdecl seekg( streampos );
    ipstream& UselessCdecl seekg( streamoff, ios::seek_dir );

    uchar UselessCdecl readByte();
    void UselessCdecl readBytes( void *, size_t );
    ushort UselessCdecl readWord();
    char * UselessCdecl readString();
    char * UselessCdecl readString( char *, unsigned );

    friend ipstream& UselessCdecl operator >> ( ipstream&, signed char& );
    friend ipstream& UselessCdecl operator >> ( ipstream&, unsigned char& );
    friend ipstream& UselessCdecl operator >> ( ipstream&, char& );
    friend ipstream& UselessCdecl operator >> ( ipstream&, signed short& );
    friend ipstream& UselessCdecl operator >> ( ipstream&, unsigned short& );
    friend ipstream& UselessCdecl operator >> ( ipstream&, signed int& );
    friend ipstream& UselessCdecl operator >> ( ipstream&, unsigned int& );
    friend ipstream& UselessCdecl operator >> ( ipstream&, signed long& );
    friend ipstream& UselessCdecl operator >> ( ipstream&, unsigned long& );
    friend ipstream& UselessCdecl operator >> ( ipstream&, float& );
    friend ipstream& UselessCdecl operator >> ( ipstream&, double& );
    friend ipstream& UselessCdecl operator >> ( ipstream&, long double& );

    friend ipstream& UselessCdecl operator >> ( ipstream&, TStreamable& );
    friend ipstream& UselessCdecl operator >> ( ipstream&, void *& );

protected:

    UselessCdecl ipstream();

    const TStreamableClass * UselessCdecl readPrefix();
    void * UselessCdecl readData( const TStreamableClass *,
                                        TStreamable * );
    void UselessCdecl readSuffix();

    const void * UselessCdecl find( P_id_type );
    void UselessCdecl registerObject( const void *adr );

private:

    TPReadObjects objs;

};

#endif  // Uses_ipstream

/* ------------------------------------------------------------------------*/
/*                                                                         */
/*   class opstream                                                        */
/*                                                                         */
/*   Base class for writing streamable objects                             */
/*                                                                         */
/* ------------------------------------------------------------------------*/

#if defined( Uses_opstream ) && !defined( __opstream )
#define __opstream

#if !defined( __IOSTREAM_H )
#include <iostream.h>
#endif  // __IOSTREAM_H


class TStreamableClass;

class opstream : virtual public pstream
{

public:

    UselessCdecl opstream( streambuf * );
    UselessCdecl ~opstream();

    streampos UselessCdecl tellp();
    opstream& UselessCdecl seekp( streampos );
    opstream& UselessCdecl seekp( streamoff, ios::seek_dir );
    opstream& UselessCdecl flush();

    void UselessCdecl writeByte( uchar );
    void UselessCdecl writeBytes( const void *, size_t );
    void UselessCdecl writeWord( ushort );
    void UselessCdecl writeString( const char * );

    friend opstream& UselessCdecl operator << ( opstream&, signed char );
    friend opstream& UselessCdecl operator << ( opstream&, unsigned char );
    friend opstream& UselessCdecl operator << ( opstream&, char );
    friend opstream& UselessCdecl operator << ( opstream&, signed short );
    friend opstream& UselessCdecl operator << ( opstream&, unsigned short );
    friend opstream& UselessCdecl operator << ( opstream&, signed int );
    friend opstream& UselessCdecl operator << ( opstream&, unsigned int );
    friend opstream& UselessCdecl operator << ( opstream&, signed long );
    friend opstream& UselessCdecl operator << ( opstream&, unsigned long );
    friend opstream& UselessCdecl operator << ( opstream&, float );
    friend opstream& UselessCdecl operator << ( opstream&, double );
    friend opstream& UselessCdecl operator << ( opstream&, long double );

    friend opstream& UselessCdecl operator << ( opstream&, TStreamable& );
    friend opstream& UselessCdecl operator << ( opstream&, TStreamable * );

protected:

    UselessCdecl opstream();

    void UselessCdecl writePrefix( const TStreamable& );
    void UselessCdecl writeData( TStreamable& );
    void UselessCdecl writeSuffix( const TStreamable& );

    P_id_type UselessCdecl find( const void *adr );
    void UselessCdecl registerObject( const void *adr );

private:

    TPWrittenObjects *objs;

};

#endif  // Uses_opstream

/* ------------------------------------------------------------------------*/
/*                                                                         */
/*   class iopstream                                                       */
/*                                                                         */
/*   Base class for reading and writing streamable objects                 */
/*                                                                         */
/* ------------------------------------------------------------------------*/

#if defined( Uses_iopstream ) && !defined( __iopstream )
#define __iopstream

#if !defined( __IOSTREAM_H )
#include <iostream.h>
#endif  // __IOSTREAM_H

class iopstream : public ipstream, public opstream
{

public:

    UselessCdecl iopstream( streambuf * );
    UselessCdecl ~iopstream();

protected:

    UselessCdecl iopstream();

};

#endif  // Uses_iopstream

/* ------------------------------------------------------------------------*/
/*                                                                         */
/*   class fpbase                                                          */
/*                                                                         */
/*   Base class for handling streamable objects on file streams            */
/*                                                                         */
/* ------------------------------------------------------------------------*/

#if defined( Uses_fpbase ) && !defined( __fpbase )
#define __fpbase

#if !defined( __FSTREAM_H )
#include <fstream.h>
#endif  // __FSTREAM_H


class fpbase : virtual public pstream
{

public:

    UselessCdecl fpbase();
    UselessCdecl fpbase( const char *, int, int = filebuf::openprot );
    UselessCdecl fpbase( int );
    UselessCdecl fpbase( int, char *, int );
    UselessCdecl ~fpbase();

    void UselessCdecl open( const char *, int, int = filebuf::openprot );
    void UselessCdecl attach( int );
    void UselessCdecl close();
    void UselessCdecl setbuf( char *, int );
    filebuf * UselessCdecl rdbuf();

private:

    filebuf buf;

};

#endif  // Uses_fpbase

/* ------------------------------------------------------------------------*/
/*                                                                         */
/*   class ifpstream                                                       */
/*                                                                         */
/*   Base class for reading streamable objects from file streams           */
/*                                                                         */
/* ------------------------------------------------------------------------*/

#if defined( Uses_ifpstream ) && !defined( __ifpstream )
#define __ifpstream

#if !defined( __IOSTREAM_H )
#include <iostream.h>
#endif  // __IOSTREAM_H

class ifpstream : public fpbase, public ipstream
{

public:

    UselessCdecl ifpstream();
    UselessCdecl ifpstream( const char *,
                      int = ios::in,
                      int = filebuf::openprot

                    );
    UselessCdecl ifpstream( int );
    UselessCdecl ifpstream( int, char *, int );
    UselessCdecl ~ifpstream();

    filebuf * UselessCdecl rdbuf();
    void UselessCdecl open( const char *,
                      int = ios::in,
                      int = filebuf::openprot

                    );

};

#endif  // Uses_ifpstream

/* ------------------------------------------------------------------------*/
/*                                                                         */
/*   class ofpstream                                                       */
/*                                                                         */
/*   Base class for writing streamable objects to file streams             */
/*                                                                         */
/* ------------------------------------------------------------------------*/

#if defined( Uses_ofpstream ) && !defined( __ofpstream )
#define __ofpstream

#if !defined( __IOSTREAM_H )
#include <iostream.h>
#endif  // __IOSTREAM_H

class ofpstream : public fpbase, public opstream
{

public:

    UselessCdecl ofpstream();
    UselessCdecl ofpstream( const char *,
                      int = ios::out,
                      int = filebuf::openprot
                    );
    UselessCdecl ofpstream( int );
    UselessCdecl ofpstream( int, char *, int );
    UselessCdecl ~ofpstream();

    filebuf * UselessCdecl rdbuf();
    void UselessCdecl open( const char *,
                      int = ios::out,
                      int = filebuf::openprot
                    );

};

#endif  // Uses_ofpstream

/* ------------------------------------------------------------------------*/
/*                                                                         */
/*   class fpstream                                                        */
/*                                                                         */
/*   Base class for reading and writing streamable objects to              */
/*   bidirectional file streams                                            */
/*                                                                         */
/* ------------------------------------------------------------------------*/

#if defined( Uses_fpstream ) && !defined( __fpstream )
#define __fpstream

#if !defined( __IOSTREAM_H )
#include <iostream.h>
#endif  // __IOSTREAM_H

class fpstream : public fpbase, public iopstream
{

public:

    UselessCdecl fpstream();
    UselessCdecl fpstream( const char *, int, int = filebuf::openprot );
    UselessCdecl fpstream( int );
    UselessCdecl fpstream( int, char *, int );
    UselessCdecl ~fpstream();

    filebuf * UselessCdecl rdbuf();
    void UselessCdecl open( const char *, int, int = filebuf::openprot );

};


#endif  // Uses_fpstream


