/*------------------------------------------------------------*/
/* filename -       tfillist.cpp                              */
/*                                                            */
/* function(s)                                                */
/*                  TFileList member functions                */
/*------------------------------------------------------------*/

/*------------------------------------------------------------*/
/*                                                            */
/*    Turbo Vision -  Version 1.0                             */
/*                                                            */
/*                                                            */
/*    Copyright (c) 1991 by Borland International             */
/*    All Rights Reserved.                                    */
/*                                                            */
/*------------------------------------------------------------*/

#define Uses_TVMemMgr
#define Uses_MsgBox
#define Uses_TFileList
#define Uses_TRect
#define Uses_TSearchRec
#define Uses_TEvent
#define Uses_TGroup
#define Uses_DirService
#include <tv.h>

#if defined(__TV4BCC__) || defined(__TV4TS__)
  #if !defined( __DIR_H )
  #include <Dir.h>
  #endif  // __DIR_H
#endif

#if !defined( __ERRNO_H )
#include <Errno.h>
#endif  // __ERRNO_H

#if !defined( __IO_H )
#include <IO.h>
#endif  // __IO_H

#if !defined( __STDIO_H )
#include <Stdio.h>
#endif  // __STDIO_H

#if !defined( __CTYPE_H )
#include <ctype.h>
#endif  // __CTYPE_H

#if !defined( __ASSERT_H )
#include <Assert.h>
#endif  // __ASSERT_H

#if defined(__TV4TS__) || defined(__TV4BCC__)
  #if !defined( __DOS_H )
  #include <Dos.h>
  #endif  // __DOS_H
#endif

#if !defined( __STRING_H )
#include <String.h>
#endif  // __STRING_H

void fexpand( char * );

TFileList::TFileList( const TRect& bounds,
                      TScrollBar *aScrollBar) :
//    TSortedListBox( bounds, 2, aScrollBar )
    TSortedListBox( bounds, 1, aScrollBar )
{
}

TFileList::~TFileList()
{
   if ( list() )
      destroy ( list() );
}

void TFileList::focusItem( short item )
{
    TSortedListBox::focusItem( item );
    message( owner, evBroadcast, cmFileFocused, list()->at(item) );
}

void TFileList::getData( void * )
{
}

void TFileList::setData( void * )
{
}

size_t TFileList::dataSize()
{
    return 0;
}

void* TFileList::getKey( const char *s )
{
static TSearchRec sR;

    if( (shiftState & 0x03) != 0 || *s == '.' )
        sR.attr = FILE_DIRECTORY;
    else
        sR.attr = 0;
    strcpy( sR.name, s );
    strupr( sR.name );
    return &sR;
}

void TFileList::getText( char *dest, short item, short maxChars )
{
	TSearchRec *f = (TSearchRec *)(list()->at(item));

	strncpy( dest, f->name, maxChars );
	dest[maxChars] = '\0';
    if( f->attr & FILE_DIRECTORY )
        strcat( dest, "\\" );
}

void TFileList::handleEvent( TEvent & event )
{
    if( event.what == evMouseDown && event.mouse.doubleClick )
        {
        event.what = evCommand;
        event.message.command = cmOK;
        putEvent( event );
        clearEvent( event );
        }
    else
        TSortedListBox::handleEvent( event );
}

void TFileList::readDirectory( const char *dir, const char *wildCard )
{
    char path[TV_MAXPATH];
    strcpy( path, dir );
    strcat( path, wildCard );
    readDirectory( path );
}

struct DirSearchRec : public TSearchRec
{

    void *operator new( size_t );

};

void *DirSearchRec::operator new( size_t sz )
{
    void *temp = ::operator new( sz );
    if( TVMemMgr::safetyPoolExhausted() )
        {
        delete temp;
        temp = 0;
        }
    return temp;
}

void TFileList::readDirectory( const char *aWildCard )
{
char path[TV_MAXPATH];
char drive[TV_MAXDRIVE];
char dir[TV_MAXPATH];
char file[TV_MAXPATH];
char ext[TV_MAXEXT];

    DirService server;
    TSearchRec tmpSearchRec;
    const unsigned findAttr = FILE_READONLY | FILE_ARCHIVED;
    strcpy( path, aWildCard );
    fexpand( path );
    fnsplit( path, drive, dir, file, ext );

    TFileCollection *fileList = new TFileCollection( 5, 5 );
    int res = server.findfirst( aWildCard, findAttr, tmpSearchRec );
    DirSearchRec *p = (DirSearchRec *)&p;
    while( p != 0 && res == 0 )
        {
        if( (tmpSearchRec.attr & FILE_DIRECTORY) == 0 )
            {
            p = new DirSearchRec;
            if( p != 0 )
                {
                memcpy( p, &tmpSearchRec, sizeof( DirSearchRec ) );
                fileList->insert( p );
                }
            }
        res = server.findnext( tmpSearchRec );
        }
    fnmerge( path, drive, dir, "*", ".*" );
    res = server.findfirst( path, FILE_DIRECTORY, tmpSearchRec );
    while( p != 0 && res == 0 )
        {
        if( (tmpSearchRec.attr & FILE_DIRECTORY) != 0 && tmpSearchRec.name[0] != '.' )
            {
            p = new DirSearchRec;
            if( p != 0 )
                {
                memcpy( p, &tmpSearchRec, sizeof( DirSearchRec ) );
                fileList->insert( p );
                }
            }
          res = server.findnext( tmpSearchRec );
        }
    if( strlen( dir ) > 1 )
        {
        p = new DirSearchRec;
        if( p != 0 )
            {
            if( server.findfirst( path, FILE_DIRECTORY, tmpSearchRec ) == 0 &&
                server.findnext( tmpSearchRec ) == 0 &&
                strcmp( tmpSearchRec.name, ".." ) == 0
              )
                {
                memcpy( p, &tmpSearchRec, sizeof( DirSearchRec ) );
                }
            else
                {
                strcpy( p->name, ".." );
                p->size = 0;
                p->seconds=0; p->minutes=0; p->hours=0;
                p->day=1; p->month=1; p->year=0;
                p->attr = FILE_DIRECTORY;
                }
            fileList->insert( p );
            }
        }

    if( p == 0 )
        messageBox( tooManyFiles, mfOKButton | mfWarning );
    newList(fileList);
    if( list()->getCount() > 0 )
        message( owner, evBroadcast, cmFileFocused, list()->at(0) );
    else
        {
        static DirSearchRec noFile;
        message( owner, evBroadcast, cmFileFocused, &noFile );
        }
};


/*
    fexpand:    reimplementation of pascal's FExpand routine.  Takes a
                relative DOS path and makes an absolute path of the form

                    drive:\[subdir\ ...]filename.ext

                works with '/' or '\' as the subdir separator on input;
                changes all to '\' on output.

*/

void squeeze( char *path )
{
    char *dest = path;
    char *src = path;
    while( *src != 0 )
        {
        if( *src != '.' )
            *dest++ = *src++;   // just copy it...
        else
            {
            src++;
            if( *src == '.' )
                {               // have a '..'
                src += 2;       // skip the following '\'
                dest--;         // back up to the previous '\'
                while( *--dest != '\\' ) // back up to the previous '\'
                    ;
                dest++;         // move to the next position
                }
            else
                {
                src++;          // skip the following '\'
                dest += 2;
                }
            }
        }
    *dest = EOS;                // zero terminator
}

void fexpand( char *rpath )
{
char path[TV_MAXPATH];
char drive[TV_MAXDRIVE];
#if 0
char dir[MAXDIR];
char file[MAXFILE];
char ext[MAXEXT];
#endif
char dir[TV_MAXPATH];
char file[TV_MAXPATH];
char ext[TV_MAXEXT];

    int flags = fnsplit( rpath, drive, dir, file, ext );
    if( (flags & DRIVE) == 0 )
        {
        drive[0] = getdisk() + 'A';
        drive[1] = ':';
        drive[2] = '\0';
        }
    drive[0] = toupper(drive[0]);
    if( (flags & DIRECTORY) == 0 || (dir[0] != '\\' && dir[0] != '/') )
        {
        char curdir[TV_MAXPATH];
        getcurdir( drive[0] - 'A' + 1, curdir );
        strcat( curdir, dir );
        if( *curdir != '\\' && *curdir != '/' )
            {
            *dir = '\\';
            strcpy( dir+1, curdir );
            }
        else
            strcpy( dir, curdir );
        }

    squeeze( dir );
    char *p = dir;
    while( (p = strchr( p, '/' )) != 0 )
        *p = '\\';
    fnmerge( path, drive, dir, file, ext );
#if defined(__TV4WATCOMDOS__) || defined (__TV4DJ__) || defined(__TV4TS__)
    strupr( path );
#endif
    strcpy( rpath, path );
}

TStreamable *TFileList::build()
{
    return new TFileList( streamableInit );
}


